/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.drawables;

import org.nfunk.jep.JEP;
import org.nfunk.jep.ParseException;

/**
 * This class provides a quick way to implement mathematical functions: parsing
 * them from strings. Just pass a string like "x^2+sin(x)" to the constructor
 * and you'll get a drawable function to add to your DefaultChart. <br />
 * The parsing is done with the JEP parser, so strings should follow JEP's
 * syntax.
 * 
 * @see org.nfunk.jep.JEP
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public class ParsedFunction extends AbstractDrawableFunction {
    /** Parser object */
    private JEP p;

    /** String to be parsed */
    private String s;

    /**
     * Constructs a new function with the given string. The string should follow
     * JEP's syntax.
     * 
     * @see org.nfunk.jep.JEP
     * @param s
     *            the string to be parsed
     * @throws ParseException
     *             if the string cannot be parsed by JEP
     */
    public ParsedFunction(String s) throws ParseException {
        super();
        this.p = new JEP();
        this.p.addStandardFunctions();
        this.p.addStandardConstants();
        this.p.setImplicitMul(true);
        this.p.addVariable("x", 0);
        this.p.parseExpression(s);
        if (this.p.hasError()) {
            throw new ParseException("Couldn't parse: " + s);
        }
        this.s = s;
    }

    /**
     * Returns the y value corresponding to the specified x.
     * 
     * @param x
     *            the abscissa
     * @return the ordinate
     */
    public double f(double x) {
        this.p.addVariable("x", x);
        return this.p.getValue();
    }

    /**
     * Returns a string describing this function.
     * 
     * @return a string describing this function
     */
    public String toString() {
        return "y=" + this.s;
    }
}